{ Program One utilizes the finite horizon undiscounted cost criterion.}
{ The model is a single server queue with geometric service times. }
{ There is a linear holding cost on the number of customers in the queue }
{ and a terminal cost equal to the holding cost on the number of remaining  }
{ customers. The arrival of customers follows a Poisson process and the }
{ controller has the option of rejecting the arriving batch }
{ (before its size is observed) and incurring a penalty cost. }

 
program FiniteHorizonQueueingModel (input, output);

{Section 3.4}

{The holding cost is linear in the buffer content.}

{There is a cost for rejecting an incoming batch.}

{The batch size follows a Poisson distribution.}

{The terminal cost is the holding cost.}

{It is always optimal to accept in N.}

 

const

UB = 50;

B = 49;

Horizon = 25;

 

type

RealList = array[0..B] of double;

IntList = array[0..UB] of integer;

 

var

n: integer;

H, RC, lambda, mu, V, NewV: double;

S, W, TempS, TempW: RealList;

OptAct: IntList;

fpout: TEXT;

file_name: string[80];

 

procedure FillInitialS;

{This fills the S array with the appropriate values for horizon zero.}

var

i: integer;

begin

for i := 0 to B do

begin

S[i] := H * (i - UB);

end;

end;

 

procedure FillInitialW;

{This fills the W array with the appropriate values for horizon zero.}

var

i, j: integer;

Term, Sum: real;

begin

for i := 0 to B do

begin

Term := 1;

Sum := i - UB;

for j := 1 to UB - i do

begin

Term := (Term * lambda) / j;

Sum := Sum + Term * (i + j - UB);

end;

W[i] := exp(-lambda) * H * Sum;

end;

end;

 

procedure CreateTempS;

{This performs one iteration, creates the TempS array and identifies the optimal action in each state in [0,N-1].}

var

i: integer;

Term, NewTerm: real;

begin

Term := W[0];

OptAct[0] := 1;

NewTerm := RC + S[0];

if NewTerm < Term then

begin

Term := NewTerm;

OptAct[0] := 2;

end;

TempS[0] := V + Term - NewV;

for i := 1 to B do

begin

Term := mu * W[i - 1] + (1 - mu) * W[i];

OptAct[i] := 1;

NewTerm := RC + mu * S[i - 1] + (1 - mu) * S[i];

if NewTerm < Term then

begin

Term := NewTerm;

OptAct[i] := 2;

end;

TempS[i] := V + H * i + Term - NewV;

end;

end;

 

procedure UpdateSandPrint;

var

i: integer;

begin

for i := 0 to B do

begin

S[i] := TempS[i];

writeln(i : 10, ' ' : 15, S[i] : 15 : 6, ' ' : 15, OptAct[i] : 10);

writeln(fpout, i : 10, ' ' : 15, S[i] : 15 : 6, ' ' : 15, OptAct[i] : 10);

end;

end;

 

procedure CreateTempW;

var

i, j: integer;

Term, Sum: real;

begin

for i := 0 to B do

begin

Term := 1;

Sum := S[i];

for j := 1 to B - i do

begin

Term := (Term * lambda) / j;

Sum := Sum + Term * S[i + j];

end;

TempW[i] := exp(-lambda) * Sum;

end;

end;

 

procedure UpdateW;

var

i: integer;

begin

for i := 0 to B do

begin

W[i] := TempW[i];

end;

end;

 

{Here is the main program.}

begin

writeln('Enter the run name.');

readln(file_name);

rewrite(fpout, file_name);

writeln(fpout, file_name);

 

writeln;

writeln(fpout);

 

writeln('Approximation level', UB);

writeln(fpout, 'Approximation level', UB);

 

writeln('Enter the positive holding cost coefficient.');

readln(H);

writeln('Holding cost coeffcient', H : 15 : 6);

writeln(fpout, 'Holding cost coeffcient', H : 15 : 6);

 

writeln('Enter the positive rejection cost.');

readln(RC);

writeln('Rejection cost', RC : 15 : 6);

writeln(fpout, 'Rejection cost', RC : 15 : 6);

 

writeln('Enter the mean batch size.');

readln(lambda);

writeln('Mean batch size', lambda : 15 : 6);

writeln(fpout, 'Mean batch size', lambda : 15 : 6);

 

writeln('Enter the mean service rate - the probability of a successful service.');

writeln('This should be a positive number <= 1.');

readln(mu);

writeln('Mean service rate', mu : 15 : 6);

writeln(fpout, 'Mean service rate', mu : 15 : 6);

 

writeln;

writeln(fpout);

 

writeln('Each state from 0 to N-1 is followed by the relative value then the optimal action,');

writeln('where 1 = accept and 2 = reject . ');

writeln(fpout, 'Each state from 0 to N-1 is followed by the relative value then the optimal action,');

writeln(fpout, 'where 1 = accept and 2 = reject . ');

 

FillInitialS;

FillInitialW;

V := H * UB;

 

for n := 1 to Horizon do

begin

writeln;

writeln(fpout);

writeln('Horizon', n : 15);

writeln(fpout, 'Horizon ', n : 15);

NewV := V + H * UB + mu * W[B];

writeln('Value function at', UB : 10, NewV : 15 : 6);

writeln(fpout, 'Value function at', UB : 10, NewV : 15 : 6);

writeln;

writeln(fpout);

CreateTempS;

UpdateSandPrint;

CreateTempW;

UpdateW;

V := NewV;

end;

close(fpout);

end.

