{ Program Two utilizes the infinite horizon discounted cost criterion. }
{ An inventory model with stochastic demand and backlogging}
{ is treated. The action at the beginning of a period is the production }
{ level for that period. The cost structure involves the revenue }
{ from the items sold during the previous period, a holding / penalty cost }
{ on current inventory, and a cost of ordering. Revenues are incorporated }
{ into the model as negative costs. The program output actually yields }
{ the maximum expected discounted profit. }

{ The program assumes linear holding and penalty costs, and linear ordering }
{ costs with a fixed set up cost. The theory presented }
{ in Chapter 5 makes it clear how to relax these assumptions. }

 

 


 

program InfiniteHorizonDiscountedInventoryModel (input, output);

 

{Chapter 5.}

{Set MinL= - MaxL. Be sure that MaxD < MaxP.}

 

const

MaxL = 20;

MinL = -20;

MaxD = 3;

MaxP = 4;

HL = 125;

 

type

RealList = array[0..MaxD] of double;

RealListOne = array[MinL..MaxL, 0..MaxP, 0..MaxD] of double;

IntList = array[MinL..MaxL, 0..MaxP, 0..MaxD] of integer;

 

var

n, W: integer;

DF, HOLD, PEN, SETUP, CPI, RPI, VF, NewVF: double;

PROB: RealList;

RV, Inter: RealListOne;

OPTACT: IntList;

fpout: TEXT;

file_name: string[80];

 

procedure FillPROB;

{This fills an array with the probabilities that from 0 to MaxD items are demanded in one period.}

var

y: integer;

 

begin

writeln('Enter', MaxD + 1 : W, ' ' : W, 'nonnegative numbers that sum to 1.');

for y := 0 to MaxD do

begin

read(PROB[y]);

writeln(fpout, 'The probability that', y : W, ' ' : W, 'items are demanded is', PROB[y] : 15 : 6);

end;

end;

 

procedure FillRV;

{This fills the array of relative values with 0.}

 

var

x, k, y: integer;

 

begin

for x := MinL to MaxL do

begin

for k := 0 to MaxP do

begin

for y := 0 to MaxD do

begin

RV[x, k, y] := 0;

end;

end;

end;

end;

 

procedure FindInter;

{This finds the minimum, identifies the optimal order, and creates an array containing the minimizing term for each state.}

 

var

x, OLDx, k, OLDk, y, OLDy: integer;

Sum, Term, NewTerm: double;

 

begin

for OLDx := MinL to MaxL do

begin

for OLDk := 0 to MaxP do

begin

for OLDy := 0 to MaxD do

begin

Inter[OLDx, OLDk, OLDy] := 0;

OPTACT[OLDx, OLDk, OLDy] := 0;

x := OLDx + OLDk - OLDy;

if (MinL <= x) and (x <= MaxL) then

begin

Sum := 0;

for y := 0 to MaxD do

begin

Sum := Sum + PROB[y] * RV[x, 0, y];

end;

Term := DF * Sum;

for k := 1 to MaxP do

begin

Sum := 0;

for y := 0 to MaxD do

begin

Sum := Sum + PROB[y] * RV[x, k, y];

end;

NewTerm := SETUP + CPI * k + DF * Sum;

if NewTerm < Term then

begin

Term := NewTerm;

OPTACT[OLDx, OLDk, OLDy] := k;

end;

end;

Inter[OLDx, OLDk, OLDy] := Term;

end;

end;

end;

end;

end;

 

procedure Update;

{This procedure updates first VF and then RV.}

 

var

x, OLDx, u, OLDk, OLDy, SOLD: integer;

INV: double;

 

begin

NewVF := DF * VF + RPI * MaxP + Inter[0, 0, 0];

for OLDx := MinL to MaxL do

begin

if OLDx < 0 then

u := -OLDx

else

u := 0;

for OLDk := 0 to MaxP do

begin

for OLDy := 0 to MaxD do

begin

x := OLDx + OLDk - OLDy;

if x < 0 then

begin

SOLD := OLDx + OLDk + u;

INV := PEN * (-x);

end;

if x >= 0 then

begin

SOLD := OLDy + u;

INV := HOLD * x;

end;

RV[OLDx, OLDk, OLDy] := DF * VF + INV + RPI * (MaxP - SOLD) + Inter[OLDx, OLDk, OLDy] - NewVF;

end;

end;

end;

end;

 

procedure Results;

{For each state this gives the optimal production size and maximum expected profit.}

 

var

x, k, y: integer;

Term: double;

 

begin

writeln('Each state triple is followed by the current inventory level, the optimal production size, and maximum expected profit.');

writeln(fpout, 'Each state triple is followed by the current inventory level, the optimal production size, and maximum expected
profit.');

writeln(' ');

writeln(fpout, ' ');

Term := (RPI * MaxP) / (1 - DF);

for x := MinL to MaxL do

begin

for k := 0 to MaxP do

begin

for y := 0 to MaxD do

begin

writeln(x : W, ' ', k : W, ' ', y : W, ' ', x + k - y : W, ' ', OPTACT[x, k, y] : W, ' ', Term - RV[x, k, y] - VF : 15 : 6);

writeln(fpout, x : W, ' ', k : W, ' ', y : W, ' ', x + k - y : W, ' ', OPTACT[x, k, y] : W, ' ', Term - RV[x, k, y] - VF : 15 : 6);

end;

end;

end;

end;

 

{Here is the main program.}

 

begin

writeln('Enter the run name.');

readln(file_name);

rewrite(fpout, file_name);

writeln(fpout, file_name);

 

writeln;

writeln(fpout);

 

writeln('The maximum inventory level is', MaxL);

writeln(fpout, 'The maximum inventory level is', MaxL);

 

writeln;

writeln(fpout);

 

writeln('The maximum backlog is', -MinL);

writeln(fpout, 'The maximum backlog is', -MinL);

 

writeln;

writeln(fpout);

 

writeln('The horizon length is', HL);

writeln(fpout, 'The horizon length is', HL);

 

writeln;

writeln(fpout);

 

writeln('The number of items demanded in one period cannot exceed', MaxD);

writeln(fpout, 'The number of items demanded in one period cannot exceed', MaxD);

 

writeln;

writeln(fpout);

 

writeln('The number of items produced in one period cannot exceed', MaxP);

writeln(fpout, 'The number of items produced in one period cannot exceed', MaxP);

 

writeln;

writeln(fpout);

 

writeln('Enter the discount factor.');

readln(DF);

writeln('The discount factor is', DF);

writeln(fpout, 'The discount factor is', DF : 15 : 6);

 

writeln;

writeln(fpout);

 

writeln('Enter the positive holding cost coefficient.');

readln(HOLD);

writeln('The holding cost coeffcient is', HOLD);

writeln(fpout, 'The holding cost coeffcient is', HOLD : 15 : 6);

 

writeln;

writeln(fpout);

 

writeln('Enter the positive penalty cost coefficient.');

readln(PEN);

writeln('The penalty cost coeffcient is', PEN);

writeln(fpout, 'The penalty cost coeffcient is', PEN : 15 : 6);

 

writeln;

writeln(fpout);

 

writeln('Enter the nonnegative set-up cost.');

readln(SETUP);

writeln('The set-up cost is', SETUP);

writeln(fpout, 'The set-up cost is', SETUP : 15 : 6);

 

writeln;

writeln(fpout);

 

writeln('Enter the positive cost per item.');

readln(CPI);

writeln('The cost per item is', CPI);

writeln(fpout, 'The cost per item is', CPI : 15 : 6);

 

writeln;

writeln(fpout);

 

writeln('Enter the positive revenue per item.');

readln(RPI);

writeln('The revenue per item is', RPI);

writeln(fpout, 'The revenue per item is', RPI : 15 : 6);

 

writeln;

writeln(fpout);

 

W := 10;

FillPROB;

FillRV;

VF := 0;

 

for n := 1 to HL do

begin

FindInter;

Update;

VF := NewVF;

end;

writeln;

writeln(fpout);

Results;

close(fpout);

 

end.
 