{ Program Four utilizes the average cost criterion. The model consists }
{ of two parallel queues each with fixed rate geometric service and }
{ linear holding cost. The system arrival process is Bernoulli and the }
{ arriving customer is routed to one of the parallel queues. The benchmark }
{ open loop control is the fixed splitting yielding minimum average cost }
{ among fixed splitting policies. The minimum average cost and optimal routing }
{ policy are computed. }

 

 
 

program RoutingModel (input, output);

 

{Section 8.7}

{This is a routing problem with two buffers.}

{There is a probability p of one packet arriving in any slot . }

{The holding cost is linear in each buffer content.}

 

const

N = 23;

{This is the approximation level. Because of the printout structure, N +1 should be divisible by 4.}

 

type

RTable = array[0..N, 0..N] of double;

IntTable = array[0..N, 0..N] of integer;

 

var

MAX, I, Z: integer;

p, H1, H2, M1, M2, epsilon, delta, Old: double;

Fun, WFun: RTable;

Act: IntTable;

fpout: TEXT;

file_name: string[80];

 

procedure InitialFun;

{This fills the Fun array with zeros.}

{The Fun array corresponds to u.}

 

var

i, j: integer;

begin

for i := 0 to N do

for j := 0 to N do

begin

Fun[i, j] := 0;

end;

end;

 

procedure CreateWFun;

{This creates the WFun array and identifies the minimizing choice of routing for each vector (i,j).}

{The WFun array corresponds to w.}

 

var

i, j, aux1, aux2, bux1, bux2: integer;

NoArr, TermOne, TermTwo, Term: double;

begin

for i := 0 to N do

for j := 0 to N do

begin

{The variables aux and bux allow us to handle boundary states without separate expressions.}

aux1 := i + 1;

if aux1 > N then

begin

aux1 := N;

end;

aux2 := j + 1;

if aux2 > N then

begin

aux2 := N;

end;

bux1 := i - 1;

if bux1 < 0 then

begin

bux1 := 0;

end;

bux2 := j - 1;

if bux2 < 0 then

begin

bux2 := 0;

end;

{NoArr corresponds to Eq.(8.12)}

NoArr := (1 - M1) * (1 - M2) * Fun[i, j] + M1 * (1 - M2) * Fun[bux1, j] + M2 * (1 - M1) * Fun[i, bux2] + M1 * M2 *
Fun[bux1, bux2];

{TermOne corresponds to (8.13)}

TermOne := (1 - M1) * (1 - M2) * Fun[aux1, j] + M1 * (1 - M2) * Fun[bux1 + 1, j] + M2 * (1 - M1) * Fun[aux1, bux2] +
M1 * M2 * Fun[bux1 + 1, bux2];

Term := TermOne;

Act[i, j] := 1;

{TermTwo is the analog of (8.13) when the packet is routed to buffer 2.}

TermTwo := (1 - M1) * (1 - M2) * Fun[i, aux2] + M1 * (1 - M2) * Fun[bux1, aux2] + M2 * (1 - M1) * Fun[i, bux2 + 1] +
M1 * M2 * Fun[bux1, bux2 + 1];

if TermTwo < TermOne then

begin

Term := TermTwo;

Act[i, j] := 2;

end;

WFun[i, j] := H1 * i + H2 * j + (1 - p) * NoArr + p * Term;

end;

end;

 

procedure UpdateFun;

{This updates the Fun array after the previous iteration.}

 

var

i, j: integer;

begin

for i := 0 to N do

for j := 0 to N do

begin

Fun[i, j] := WFun[i, j] - WFun[0, 0];

end;

end;

 

procedure WriteOptAct;

{This writes a state followed by the optimal routing in that state. The output is organized in four columns.}

 

var

i, group: integer;

begin

writeln('Each state vector (i,j) is followed by the optimal action, namely 1 or 2.');

writeln(fpout, 'Each state vector (i,j) is followed by the optimal action, namely 1 or 2.');

writeln(' ');

writeln(fpout, ' ');

for i := 0 to N do

begin

for group := 0 to (N - 3) div 4 do

begin

writeln(i, 4 * group : Z, ' ' : Z, Act[i, 4 * group] : Z, ' ' : Z, i, 4 * group + 1 : Z, ' ' : Z, Act[i, 4 * group + 1] : Z, ' ' : Z, i, 4 *
group + 2 : Z, ' ' : Z, Act[i, 4 * group + 2] : Z, ' ' : Z, i, 4 * group + 3 : Z, ' ' : Z, Act[i, 4 * group + 3] : Z);

writeln(fpout, i, 4 * group : Z, ' ' : Z, Act[i, 4 * group] : Z, ' ' : Z, i, 4 * group + 1 : Z, ' ' : Z, Act[i, 4 * group + 1] : Z, ' ' : Z, i, 4
* group + 2 : Z, ' ' : Z, Act[i, 4 * group + 2] : Z, ' ' : Z, i, 4 * group + 3 : Z, ' ' : Z, Act[i, 4 * group + 3] : Z);

end;

writeln;

writeln(fpout);

end;

end;

 

{Here is the main program.}

 

begin

writeln('Enter the run name.');

readln(file_name);

rewrite(fpout, file_name);

writeln(fpout, file_name);

 

writeln;

writeln(fpout);

 

writeln('The approximation level is', N);

writeln(fpout, 'The approximation level is', N);

 

writeln('Enter the probability of a packet arrival.');

readln(p);

writeln(fpout, 'The probability of a packet arrival is', p : 15 : 6);

 

writeln('Enter the positive holding cost coefficient for station one.');

readln(H1);

writeln(fpout, 'The holding cost coeffcient for station one is', H1 : 15 : 6);

 

writeln('Enter the positive holding cost coefficient for station two');

readln(H2);

writeln(fpout, 'The holding cost coeffcient for station two is', H2 : 15 : 6);

 

writeln('You will now enter the service rates.');

writeln('They must be strictly between 0 and 1, and their sum must exceed p.');

 

writeln(fpout, 'You will now enter the service rates.');

writeln(fpout, 'They must be strictly between 0 and 1, and their sum must exceed p.');

 

writeln('Enter the service rate for station one.');

readln(M1);

writeln(fpout, 'The service rate for station one is', M1 : 15 : 6);

 

writeln('Enter the service rate for station two.');

readln(M2);

writeln(fpout, 'The service rate for station two is', M2 : 15 : 6);

 

writeln('Enter the tolerance. This is a small positive number.');

readln(epsilon);

writeln(fpout, 'The tolerance is', epsilon);

 

writeln('Enter the maximum number of iterations allowed.');

readln(MAX);

writeln(fpout, 'The maximum number of iterations allowed is', MAX);

 

writeln;

writeln(fpout);

 

InitialFun;

Old := 1;

I := 1;

delta := 1;

Z := 4;

 

while (delta > epsilon) and (I < MAX) do

begin

CreateWFun;

delta := abs(WFun[0, 0] - Old);

Old := WFun[0, 0];

UpdateFun;

I := I + 1;

end; {while}

 

{We have now completed the calculations. We will print out the results.}

 

writeln('The number of iterations is: ', I);

writeln(fpout, 'The number of iterations is: ', I);

 

writeln('The value of delta is:', delta : 15 : 12);

writeln(fpout, 'The value of delta is:', delta : 15 : 12);

 

writeln('The approximate average cost is:', WFun[0, 0] : 15 : 6);

writeln(fpout, 'The approximate average cost is:', WFun[0, 0] : 15 : 6);

 

writeln;

writeln(fpout);

 

WriteOptAct; {modified to also output to file}

close(fpout);

 

end.