{ Program Five ultilzes the average cost criterion. The model is a }
{ single server queue with Bernoulli customer arrivals and linear }
{ holding costs. The actions are the choices of deterministic service }
{ times of either one, two, or three slots. The service time }
{ choice cannot be changed for the duration of a service. }

 


program AverageCostDeterministicServiceQueue (input, output);

 

{Section 9.7}

{This is a single server queue with deterministic service times of 1, 2, or 3 slots being the actions.}

{There is a probability p of a single packet arriving in any slot.}

{The excess probability is sent to state N.}

{Because the print out is organized in two columns, we must choose N even.}

 

const

N = 40;

 

type

RealList = array[0..N] of double;

RealLista = array[1..N] of double;

RealListb = array[1..3] of double;

IntList = array[1..N] of integer;

 

var

MAX, I, S: integer;

p, H, epsilon, Old, delta: double;

Fun, WFun: RealList;

Fun2, WFun2, Fun31, WFun31, Fun32, WFun32: RealLista;

CostAct: RealListb;

OptAct: IntList;

fpout: TEXT;

file_name: string[80];

 

procedure FillCostAct;

{This creates the array of service costs.}

{These are nonnegative decreasing numbers.}

 

var

i: integer;

begin

writeln('Enter three strictly decreasing positive numbers.');

writeln(fpout, 'The service times are 1, 2, 3, with respective costs:');

for i := 1 to 3 do

begin

read(CostAct[i]);

writeln(fpout, CostAct[i] : 15 : 6);

end;

end;

 

procedure InitialFun;

{This fills the Fun, Fun2, Fun31, and Fun32 arrays with zeros. These are the various u arrays.}

 

var

i: integer;

begin

Fun[0] := 0;

for i := 1 to N do

begin

Fun[i] := 0;

Fun2[i] := 0;

Fun31[i] := 0;

Fun32[i] := 0;

end;

end;

 

procedure CreateWFunNoAct;

{This creates the W arrays corresponding to w(i,2,1), w(i,3,1), and w(i,3,2).}

 

var

i, aux: integer;

begin

for i := 1 to N do

begin

begin

aux := i + 1;

if aux > N then

aux := N;

end;

WFun2[i] := H * i + p * Fun[i] + (1 - p) * Fun[i - 1];

WFun32[i] := WFun2[i];

WFun31[i] := H * i + p * Fun32[aux] + (1 - p) * Fun32[i];

end;

end;

 

procedure CreateWFun;

{This creates W of the states i > 0.}

{The minimizing service time is identified for i > 1. }

 

var

i, aux: integer;

Term, SecondTerm, ThirdTerm: double;

begin

WFun[0] := p * Fun[1] + (1 - p) * Fun[0];

for i := 1 to N do

begin

aux := i + 1;

if aux > N then

begin

aux := N;

end;

Term := CostAct[1] + p * Fun[i] + (1 - p) * Fun[i - 1];

OptAct[i] := 1;

SecondTerm := CostAct[2] + p * Fun2[aux] + (1 - p) * Fun2[i];

if SecondTerm < Term then

begin

Term := SecondTerm;

OptAct[i] := 2;

end;

ThirdTerm := CostAct[3] + p * Fun31[aux] + (1 - p) * Fun31[i];

if ThirdTerm < Term then

begin

Term := ThirdTerm;

OptAct[i] := 3;

end;

WFun[i] := H * i + Term;

end;

end;

 

procedure UpdateFun;

{This updates the arrays.}

 

var

i: integer;

begin

for i := 0 to N do

begin

Fun[i] := WFun[i] - WFun[0];

end;

for i := 1 to N do

begin

Fun2[i] := WFun2[i] - WFun[0];

Fun31[i] := WFun31[i] - WFun[0];

Fun32[i] := WFun32[i] - WFun[0];

end;

end;

 

procedure WriteOptAct;

{This writes a state, at an epoch, followed by the optimal service choice in that state.}

{The output is organized in two columns . }

 

var

group: integer;

begin

writeln('Each state, at an epoch, is followed by the optimal action, namely 1, 2, or 3, in that state.');

writeln(fpout, 'Each state, at an epoch, is followed by the optimal action, namely 1, 2, or 3, in that state.');

writeln(' ');

writeln(fpout, ' ');

for group := 1 to N div 2 do

begin

writeln(2 * group - 1 : S, ' ' : S, OptAct[2 * group - 1] : S, ' ' : S, 2 * group : S, ' ' : S, OptAct[2 * group] : S);

writeln(fpout, 2 * group - 1 : S, ' ' : S, OptAct[2 * group - 1] : S, ' ' : S, 2 * group : S, ' ' : S, OptAct[2 * group] : S);

end;

end;

 

{Here is the main program.}

 

begin

writeln('Enter the run name.');

readln(file_name);

rewrite(fpout, file_name);

writeln(fpout, file_name);

 

writeln;

writeln(fpout);

 

writeln('The approximation level is', N);

writeln(fpout, 'The approximation level is', N);

 

writeln('Enter the probability of a packet arrival. This is a number strictly between 0 and 1.');

readln(p);

writeln(fpout, 'The probability of a packet arrival is', p : 15 : 6);

 

writeln('Enter the positive holding cost coefficient.');

readln(H);

writeln(fpout, 'The holding cost coeffcient is', H : 15 : 6);

 

writeln('Enter the tolerance. This is a small positive number.');

readln(epsilon);

writeln(fpout, 'The tolerance level is', epsilon : 15 : 6);

 

FillCostAct;

 

writeln('Enter the maximum number of iterations allowed.');

readln(MAX);

writeln(fpout, 'The maximum number of iterations allowed is', MAX);

 

writeln;

writeln(fpout);

 

InitialFun;

 

Old := 1;

delta := 1;

I := 1;

S := 12;

 

while (delta > epsilon) and (I < MAX) do

begin

CreateWFunNoAct;

CreateWFun;

delta := abs(WFun[0] - Old);

Old := WFun[0];

UpdateFun;

I := I + 1;

end; {while}

 

 

writeln('The number of iterations is: ', I);

writeln(fpout, 'The number of iterations is: ', I);

 

writeln('The value of delta is:', delta : 15 : 10);

writeln(fpout, 'The value of delta is:', delta : 15 : 10);

 

writeln('The approximate average cost is:', WFun[0] : 15 : 6);

writeln(fpout, 'The approximate average cost is:', WFun[0] : 15 : 6);

 

writeln;

writeln(fpout);

 

WriteOptAct; {modified to also output to file}

close(fpout);

 

end.