
{ Program Six utilizes the average cost criterion. The model is a }
{ single server queue with Bernoulli customer arrivals and linear }
{ holding costs. The actions are the choices of geometric service rates. }
{ A rate selection is made when a service begins and must }
{ remain in effect throughout the service. }

 

 

 

program AverageCostGeometricServiceQueue (input, output);

 

{Section 9.8}

{This is a single server queue with K geometric service times as actions.}

{The service rate choice for a given customer remains in effect throughout the service.}

{There is a probability p of a single customer arriving in any slot.}

{The excess probability is sent to state N and no service can last more than L slots.}

{The holding cost is linear in the buffer contents.}

{The largest service rate must exceed the customer arrival rate p.}

 

const

N = 48;

{ Because of the way the printout is structured, N must be even.}

L = 8;

K = 3;

 

type

RealList = array[0..N] of double;

RealTable = array[1..N, 1..K, 1..L] of double;

RealLista = array[1..K] of double;

IntList = array[1..N] of integer;

 

var

MAX, I, S: integer;

p, H, epsilon, Old, delta: double;

Fun, WFun: RealList;

Ser, WSer: RealTable;

Rate, CostRate: RealLista;

OptAct: IntList;

fpout: TEXT;

file_name: string[80];

 

procedure FillRate;

{This fills the Rate array with the service rates.}

 

var

j: integer;

begin

writeln('Enter', K, ' positive strictly increasing rates less than 1. The largest rate must exceed p.');

writeln(fpout, 'The service rates are:');

for j := 1 to K do

begin

read(Rate[j]);

writeln(fpout, Rate[j] : 15 : 6);

end;

end;

 

procedure FillCostRate;

{This fills the CostRate array with the costs of the service rates.}

 

var

j: integer;

begin

writeln('Enter', K, ' positive strictly increasing numbers.');

writeln(fpout, 'The costs of the rates are:');

for j := 1 to K do

begin

read(CostRate[j]);

writeln(fpout, CostRate[j] : 15 : 6);

end;

end;

 

procedure InitialFunandSer;

{This fills the Fun and Ser arrays with zeros.}

{Fun corresponds to u of the decision epochs and Ser to u of the service in progress states.}

 

var

i, j, r: integer;

begin

for i := 0 to N do

begin

Fun[i] := 0;

end;

for i := 1 to N do

for j := 1 to K do

for r := 1 to L do

begin

Ser[i, j, r] := 0;

end;

end;

 

procedure CreateWSer;

{This creates W of the service in progress states. This corresponds to the w arrays.}

 

var

i, j, r, aux: integer;

begin

for i := 1 to N do

for j := 1 to K do

for r := 1 to L - 1 do

begin

aux := i + 1;

if aux > N then

begin

aux := N;

end;

WSer[i, j, r] := H * i + Rate[j] * (1 - p) * Fun[i - 1] + Rate[j] * p * Fun[i] + (1 - Rate[j]) * p * Ser[aux, j, r + 1] + (1 -
Rate[j]) * (1 - p) * Ser[i, j, r + 1];

end;

for i := 1 to N do

for j := 1 to K do

begin

WSer[i, j, L] := H * i + (1 - p) * Fun[i - 1] + p * Fun[i];

end;

end;

 

procedure CreateWFun;

{This creates W of the states i > 0.}

{The minimizing service rate is identified for i > 1. }

 

var

i, j, aux: integer;

Term, NextTerm: double;

begin

WFun[0] := p * Fun[1] + (1 - p) * Fun[0];

for i := 1 to N do

begin

aux := i + 1;

if aux > N then

begin

aux := N;

end;

Term := CostRate[1] + Rate[1] * (1 - p) * Fun[i - 1] + Rate[1] * p * Fun[i] + (1 - Rate[1]) * p * Ser[aux, 1, 1] + (1 -
Rate[1]) * (1 - p) * Ser[i, 1, 1];

OptAct[i] := 1;

for j := 2 to K do

begin

NextTerm := CostRate[j] + Rate[j] * (1 - p) * Fun[i - 1] + Rate[j] * p * Fun[i] + (1 - Rate[j]) * p * Ser[aux, j, 1] + (1 -
Rate[j]) * (1 - p) * Ser[i, j, 1];

if NextTerm < Term then

begin

Term := NextTerm;

OptAct[i] := j;

end;

end;

WFun[i] := H * i + Term;

end;

end;

 

procedure UpdateFunandSer;

{This updates the arrays.}

 

var

i, j, r: integer;

begin

for i := 0 to N do

begin

Fun[i] := WFun[i] - WFun[0];

end;

for i := 1 to N do

for j := 1 to K do

for r := 1 to L do

begin

Ser[i, j, r] := WSer[i, j, r] - WFun[0];

end;

end;

 

procedure WriteOptAct;

{This writes a state, at an epoch, followed by the optimal service choice in that state.}

{The output is organized in two columns . }

 

var

group: integer;

begin

writeln('Each epoch state is followed by the optimal action in that state.');

writeln(fpout, 'Each epoch state is followed by the optimal action in that state.');

writeln(' ');

writeln(fpout, ' ');

for group := 1 to N div 2 do

begin

writeln(2 * group - 1 : S, ' ' : S, OptAct[2 * group - 1] : S, ' ' : S, 2 * group : S, ' ' : S, OptAct[2 * group] : S);

writeln(fpout, 2 * group - 1 : S, ' ' : S, OptAct[2 * group - 1] : S, ' ' : S, 2 * group : S, ' ' : S, OptAct[2 * group] : S);

end;

end;

 

{Here is the main program.}

 

begin

writeln('Enter the run name.');

readln(file_name);

rewrite(fpout, file_name);

writeln(fpout, file_name);

 

writeln;

writeln(fpout);

 

writeln('The approximation level is', N);

writeln(fpout, 'The approximation level is', N);

writeln('The service time truncation level is', L);

writeln(fpout, 'The service time truncation level is', L);

writeln('The number of service rates is', K);

writeln(fpout, 'The number of service rates is', K);

 

writeln('Enter the probability of a packet arrival.');

readln(p);

writeln(fpout, 'The probability of a packet arrival is', p : 15 : 6);

 

writeln('Enter the positive holding cost coefficient.');

readln(H);

writeln(fpout, 'The holding cost coeffcient is', H : 15 : 6);

 

writeln('Enter the tolerance. This is a small positive number.');

readln(epsilon);

writeln(fpout, 'The tolerance is', epsilon);

 

FillRate;

FillCostRate;

 

writeln('Enter the maximum number of iterations allowed.');

readln(MAX);

writeln(fpout, 'The maximum number of iterations allowed is', MAX);

 

writeln;

writeln(fpout);

 

InitialFunandSer;

 

Old := 1;

delta := 1;

I := 1;

S := 8;

 

while (delta > epsilon) and (I < MAX) do

begin

CreateWSer;

CreateWFun;

delta := abs(WFun[0] - Old);

Old := WFun[0];

UpdateFunandSer;

I := I + 1;

end; {while}

 

writeln('The number of iterations is: ', I);

writeln(fpout, 'The number of iterations is: ', I);

 

writeln('The value of delta is:', delta : 15 : 10);

writeln(fpout, 'The value of delta is:', delta : 15 : 10);

 

writeln('The approximate average cost is:', WFun[0] : 15 : 6);

writeln(fpout, 'The approximate average cost is:', WFun[0] : 15 : 6);

 

writeln;

writeln(fpout);

 

WriteOptAct; {modified to also output to file}

 

close(fpout);

 

end.